/* interpolation/gsl_polynomial.h
 * 
 * Copyright (C) 2001 DAN, HO-JIN
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GSL_POLYNOMIAL_H__
#define __GSL_POLYNOMIAL_H__
#include <stdlib.h>
#include <gsl/gsl_interp.h>
#include "interp_poly.h"

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS		/* empty */
# define __END_DECLS		/* empty */
#endif

__BEGIN_DECLS
/* general interpolation object */
  typedef struct
{
  gsl_interp *interp;
  double *x;
  double *y;
  size_t size;
}
gsl_polynomial;

gsl_polynomial *gsl_polynomial_alloc (const gsl_interp_type * T, size_t n);

int
gsl_polynomial_init (gsl_polynomial * polynomial, const double xa[],
		     const double ya[], size_t size);


int
gsl_polynomial_eval_e (const gsl_polynomial * polynomial, double x, double *y);

double gsl_polynomial_eval (const gsl_polynomial * polynomial, double x);

int
gsl_polynomial_eval_deriv_e (const gsl_polynomial * polynomial, double x, double *y);

double
gsl_polynomial_eval_deriv (const gsl_polynomial * polynomial, double x);

int
gsl_polynomial_eval_deriv2_e (const gsl_polynomial * polynomial, double x, double *y);

double
gsl_polynomial_eval_deriv2 (const gsl_polynomial * polynomial, double x);

int
gsl_polynomial_eval_integ_e (const gsl_polynomial * polynomial, 
    double a, double b, double *y);

double
gsl_polynomial_eval_integ (const gsl_polynomial * polynomial, double a,
			   double b);
     void gsl_polynomial_free (gsl_polynomial * polynomial);

__END_DECLS
#endif /* __GSL_POLYNOMIAL_H__ */
