/* interpolation/interp_poly.h
 * 
 * Copyright (C) 2001 DAN, HO-JIN
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_vector.h>

/* for ctags ref. 
 * gsl_interp_alloc
 * gsl_spline_alloc
 * gsl_interp
 * cspline_state_t
 * gsl_interp_cspline
 * gsl_spline
 */

void *polynomial_alloc (size_t size);

int polynomial_init (void *vstate,
		     const double xa[], const double ya[], size_t size);

int polynomial_eval (const void *vstate,
		     const double xa[], const double ya[], size_t size,
		     double x, gsl_interp_accel * acc, double *y);

int polynomial_taylor (const void *vstate, const double x[], const double f[],
		       size_t size, double x_0, double *coef);

int polynomial_deriv (const void *vstate,
		      const double xa[], const double ya[], size_t size,
		      double x, gsl_interp_accel * acc, double *y);

int polynomial_deriv2 (const void *vstate,
		       const double xa[], const double ya[], size_t size,
		       double x, gsl_interp_accel * acc, double *y);

int polynomial_integ (const void *vstate, const double xa[],
		      const double ya[], size_t size, gsl_interp_accel * acc,
		      double a, double b, double *result);

void polynomial_free (void *vstate);
