#include <stdio.h>
#include "gsl_polynomial.h"
#include "interp_poly.h"

double if1(double a, double b)
{
  double x = b;
  double result = 0.;
  result = -5.*x - 3./2.*x*x +6./3.*pow(x,3) + 1./4.*pow(x,4);
  x = a;
  result -= (-5.*x - 3./2.*x*x +6./3.*pow(x,3) + 1./4.*pow(x,4));
  return result;
}

double f1(double x, double*y)
{
  return *y = -5.-3.*x+6.*x*x+pow(x,3);
}

double df1(double x, double*dy)
{
  return *dy = -3. + 12.*x + 3.*x*x;
}

double ddf1(double x, double* ddy)
{
  return *ddy = 12. + 6.*x;
}

double if2(double a, double b)
{
  double x=b, result=0.;
  result = 1./9.*pow(x,9) - 1./2.*pow(x,2) + 3.4*x;
  x = a;
  result -= (1./9.*pow(x,9) - 1./2.*pow(x,2) + 3.4*x);
  return result;
}

double f2(double x, double*y)
{
  return *y = pow (x, 8) - x + 3.4;
}

double df2(double x, double*dy)
{
  return *dy = 8.*pow(x,7) - 1.;
}

double ddf2(double x, double* ddy)
{
  return *ddy = 56.*pow(x,6);
}

extern const gsl_interp_type *gsl_interp_polynomial;

int
main ()
{
  int i;
  double x[9] = { -1.414214, 1.449138, 1.483240, 1.516575, 1.5491932, 
    2.1, 2.2, 2.3, 2.4 };
  double y1[4], y2[9];
  size_t size1 = 4, size2 = 9;
  gsl_polynomial *test1, *test2;
  double testpt = 1.2, exact, a=-0.5, b=0.1;

  for (i = 0; i < size1; i++)
    {
      f1(x[i], y1+i);
    }
  for (i = 0; i < size2; i++)
    {
      f2(x[i], y2+i);
    }

  test1 = gsl_polynomial_alloc(gsl_interp_polynomial, size1);
  gsl_polynomial_init(test1, x, y1, size1);
  printf("testing...prob1 [Eval       ] : error = %g\n", 
      gsl_polynomial_eval(test1, testpt) - f1(testpt, &exact));
  printf("testing...prob1 [Eval_deriv ] : error = %g\n", 
      gsl_polynomial_eval_deriv(test1, testpt) - df1(testpt, &exact));
  printf("testing...prob1 [Eval_deriv2] : error = %g\n", 
      gsl_polynomial_eval_deriv2(test1, testpt) - ddf1(testpt, &exact));
  printf("testing...prob1 [Eval_integ ] : error = %g\n", 
      gsl_polynomial_eval_integ(test1, a, b) - if1(a, b));


  test2 = gsl_polynomial_alloc(gsl_interp_polynomial, size2);
  gsl_polynomial_init(test2, x, y2, size2);
  printf("testing...prob2 [Eval       ] : error = %g\n", 
      gsl_polynomial_eval(test2, testpt) - f2(testpt, &exact));
  printf("testing...prob2 [Eval_deriv ] : error = %g\n", 
      gsl_polynomial_eval_deriv(test2, testpt) - df2(testpt, &exact));
  printf("testing...prob2 [Eval_deriv2] : error = %g\n", 
      gsl_polynomial_eval_deriv2(test2, testpt) - ddf2(testpt, &exact));
  printf("testing...prob2 [Eval_integ ] : error = %g\n", 
      gsl_polynomial_eval_integ(test2, a, b) - if2(a, b));

  gsl_polynomial_free(test1);
  gsl_polynomial_free(test2);

  return 0;
}
